; ****************************************************************************
; "MOS6502" - MOS 6502 X-assembler
;
; Written by De Zhi Tan <dzt@bttr-software.de>
; for BTTR Software
;
; Copyright (C) 1997-2001 BTTR Software
; [Under GNU GENERAL PUBLIC LICENSE]
;
; This program is free software; you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation; either version 2 of the License, or
; (at your option) any later version.
;
; This program is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with this program; if not, write to the Free Software
; Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
; ****************************************************************************

[BITS 16]
[ORG 0x100]

_OPCODE_        equ 01
_A_REG_         equ 02
_X_REG_         equ 03
_Y_REG_         equ 04
_NUMBER_        equ 05
_STRING_        equ 06
_CONDITION_     equ 07+50
_DIRECITVE_     equ 08+50
_POUND_         equ 09+50
_PLUS_          equ 10+50
_MINUS_         equ 11
_MUL_           equ 12
_UNDIV_         equ 13+50
_COMPLI_        equ 14
_COMPLI_PLUS_   equ 15
_COMPLI_MINUS_  equ 16
_AND_           equ 17
_XOR_           equ 18
_OR_            equ 19
_EQUAL_         equ 20
_COMMA_         equ 21
_COLON_         equ 22
_LPARA_         equ 23
_RPARA_         equ 24
_LBRACK_        equ 25
_RBRACK_        equ 26
_LBRACE_        equ 27
_RBRACE_        equ 28
_LSHIFT_        equ 29
_RSHIFT_        equ 30
_SYMBOL_        equ 31
_END_           equ 196
_INVALID_       equ 255

INIT:
cld
mov word [SOURCE_FILE_POS],0
mov word [SOURCE_FILE_POS+2],cs
add word [SOURCE_FILE_POS+2],0x1000
mov byte [PASS],'1'
mov byte [BSS],0
mov byte [DEF_CONDITION],0
mov byte [IGNORE],0
mov word [TIMES_COUNT],1
mov word [LABEL_OFFSET],0
mov word [LABEL_OFFSET+2],0
mov byte [LABEL_CONTENT],0
mov byte [LABEL_GLOBAL],0
mov si,TITLE
call DIS_PROC

CMD_LINE:
mov si,0x81
.L1:
cmp byte [si],13
je .ERR
cmp byte [si],32
jne .OK1
inc si
jmp .L1
.OK1:
mov word [SOURCE_FILE_NAME],si
inc si
.L2:
cmp byte [si],13
je .ERR
cmp byte [si],32
je .OK2
inc si
jmp .L2
.OK2:
mov byte [si],0
inc si
.L3:
cmp byte [si],13
je .ERR
cmp byte [si],32
jne .OK3
inc si
jmp .L3
.OK3:
mov word [BINARY_FILE_NAME],si
inc si
.L4:
cmp byte [si],13
je .OK4
cmp byte [si],32
je .OK4
inc si
jmp .L4
.OK4:
mov byte [si],0
jmp .END
.ERR:
jmp ASM_ERROR.00
.END:

ASM_OPEN_SOURCE_FILE:
mov ax,0x3d00
mov dx,word [SOURCE_FILE_NAME]
int 0x21
jnc .END
.ERR:
jmp ASM_ERROR.01
.END:
mov word [SOURCE_FILE_HANDLE],ax

ASM_CHECK_SOURCE_FILE_SIZE:
mov ax,0x4202
mov bx,word [SOURCE_FILE_HANDLE]
mov cx,0
mov dx,0
int 0x21
mov word [SOURCE_FILE_SIZE],ax
mov word [SOURCE_FILE_SIZE+2],dx
mov ax,0x1000
mul dx
or dx,dx
jnz .ERR
mov dx,cs
add ax,dx
jc .ERR
add ax,0x2000
jc .ERR
cmp ax,0xa000
jae .ERR
jmp .END
.ERR:
jmp ASM_ERROR.02
.END:
mov ax,0x4200
mov bx,word [SOURCE_FILE_HANDLE]
mov cx,0
mov dx,0
int 0x21

ASM_DETERMINE_SYMBOL_TABLE_LOCATION:
mov word [LABEL_SEG],cs
add word [LABEL_SEG],0x1000
mov ax,word [SOURCE_FILE_SIZE+2]
mov dx,0x1000
mul dx
add word [LABEL_SEG],ax
mov ax,word [SOURCE_FILE_SIZE]
mov dx,0
mov cx,16
div cx
stc
adc word [LABEL_SEG],ax

ASM_CREATE_BIN_FILE:
mov ah,0x3c
mov dx,word [BINARY_FILE_NAME]
mov cx,0
int 0x21
jnc .END
.ERR:
jmp ASM_ERROR.03
.END:
mov word [BINARY_FILE_HANDLE],ax

ASM_LOAD_SOURCE_FILE:
mov ax,cs
add ax,0x1000
mov ds,ax
.LOOP:
mov ah,0x3f
es mov bx,word [SOURCE_FILE_HANDLE]
mov cx,32768
mov dx,0
int 0x21
jc .ERR
cmp ax,32768
jne .END
mov ax,ds
add ax,32768/16
mov ds,ax
jmp .LOOP
.ERR:
push cs
pop ds
jmp ASM_ERROR.04
.END:
mov ax,cs
mov ds,ax

INITIALIZE_BUFFER:
mov di,BUFFER
mov cx,40
mov ax,''
rep stosw

INITIALIZE_SYMBOL_TABLE:
mov es,word [LABEL_SEG]
mov di,0
mov cx,40
mov ax,''
rep stosw
mov ax,cs
mov es,ax

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;      STATEMENT FETCHER
STATEMENT_FETCHER:
lds si,[SOURCE_FILE_POS]
mov di,STATEMENT_BUF
mov cx,128
rep movsw
mov ax,cs
mov ds,ax
mov al,0xa
mov cx,256
mov di,STATEMENT_BUF
repne scasb
jcxz .ERR
mov cx,di
sub cx,STATEMENT_BUF
sub word [SOURCE_FILE_SIZE],cx
sbb word [SOURCE_FILE_SIZE+2],0
add word [SOURCE_FILE_POS],cx
cmp word [SOURCE_FILE_POS],32768
jb .OK
sub word [SOURCE_FILE_POS],32768
add word [SOURCE_FILE_POS+2],32768/16
.OK:
jmp .END
.ERR:
jmp ASM_ERROR.10
.END:
mov word [BUFFER_POINTER],BUFFER+80
mov word [STATEMENT_POINTER],STATEMENT_BUF
mov byte [LABEL_CONTENT],0
mov word [TIMES_COUNT],1
mov byte [LEVEL],0

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;      STATEMENT SCANNER
STATEMENT_SCANNER:
call RESET_TYPE
call RESET_VALUE
call RESET_TOKEN_BUF
mov si,word [STATEMENT_POINTER]
mov di,TOKEN_BUF
.LOOP:
lodsb
call UPDATE_STATEMENT_POINTER
cmp al,32
je TOK_SPACE
cmp al,9
je TOK_SPACE
cmp al,'#'
je TOK_POUND
cmp al,'+'
je TOK_PLUS
cmp al,'-'
je TOK_NINUS
cmp al,'*'
je TOK_MUL
cmp al,'/'
je TOK_DIVID
cmp al,'~'
je TOK_COMPLI
cmp al,'&'
je TOK_AND
cmp al,'|'
je TOK_OR
cmp al,'^'
je TOK_XOR
cmp al,'='
je TOK_EQUAL
cmp al,','
je TOK_COMMA
cmp al,':'
je TOK_COLON
cmp al,'('
je TOK_LPARA
cmp al,')'
je TOK_RPARA
cmp al,'['
je TOK_LBRACKET
cmp al,']'
je TOK_RBRACKET
cmp al,'{'
je TOK_LBRACE
cmp al,'}'
je TOK_RBRACE
cmp al,'<'
je TOK_SHIFT
cmp al,'>'
je TOK_SHIFT
cmp al,'"'
je TOK_QUOTE
cmp al,"'"
je TOK_QUOTE
cmp al,';'
je TOK_COMMENT
cmp al,0xd
je TOK_LF
cmp al,0xa
je TOK_CR
stosb
cmp byte [TOKEN_BUF+63],0
jne .ERR
jmp .LOOP
.ERR:
jmp ASM_ERROR.15
TOK_SPACE:
jmp TOKEN_SPACE
TOK_PLUS:
TOK_NINUS:
TOK_MUL:
TOK_DIVID:
TOK_COMPLI:
TOK_AND:
TOK_OR:
TOK_XOR:
TOK_EQUAL:
TOK_COMMA:
TOK_COLON:
TOK_LPARA:
TOK_RPARA:
TOK_LBRACKET:
TOK_RBRACKET:
TOK_LBRACE:
TOK_RBRACE:
TOK_POUND:
jmp TOKEN_PROC
TOK_SHIFT:
jmp TOKEN_SHIFT
TOK_QUOTE:
jmp TOKEN_QUOTE
TOK_COMMENT:
jmp TOKEN_COMMENT
TOK_LF:
jmp TOKEN_LF
TOK_CR:
jmp TOKEN_CR
TOKEN_SPACE:                           ;SPACE
cmp byte [TOKEN_BUF],0
jne .END
jmp STATEMENT_SCANNER.LOOP
.END:
jmp STATEMENT_SCANNER_END
TOKEN_PROC:
cmp byte [TOKEN_BUF],0
jne .BACKWARD
mov byte [TOKEN_BUF],al
jmp .END
.BACKWARD:
dec si
call UPDATE_STATEMENT_POINTER
.END:
jmp STATEMENT_SCANNER_END
TOKEN_SHIFT:                           ;SHIFT
cmp byte [TOKEN_BUF],0
jne .BACKWARD
mov byte [TOKEN_BUF],al
cmp byte [si],al
jne .ERR
mov byte [TOKEN_BUF+1],al
inc si
call UPDATE_STATEMENT_POINTER
jmp .END
.BACKWARD:
dec si
call UPDATE_STATEMENT_POINTER
.END:
jmp STATEMENT_SCANNER_END
.ERR:
jmp ASM_ERROR.30
TOKEN_QUOTE:                           ;QUOTE
cmp byte [TOKEN_BUF],0
jne .BACKWARD
mov ah,al
mov di,TOKEN_BUF
stosb
mov cx,62
.LOOP:
lodsb
call UPDATE_STATEMENT_POINTER
cmp al,0xd
je .ERR1
stosb
cmp al,ah
je .END
loop .LOOP
jmp .ERR2
.BACKWARD:
dec si
call UPDATE_STATEMENT_POINTER
.END:
jmp STATEMENT_SCANNER_END
.ERR1:
jmp ASM_ERROR.40
.ERR2:
jmp ASM_ERROR.15
TOKEN_COMMENT:                         ;COMMENT
cmp byte [TOKEN_BUF],0
jne .BACKWARD
.LOOP:
lodsb
cmp al,0xd
jne .LOOP
call UPDATE_STATEMENT_POINTER
jmp STATEMENT_SCANNER
.BACKWARD:
dec si
call UPDATE_STATEMENT_POINTER
jmp STATEMENT_SCANNER_END
TOKEN_LF:                              ;LINE FEED
cmp byte [TOKEN_BUF],0
jne STATEMENT_SCANNER_END
jmp STATEMENT_SCANNER.LOOP
TOKEN_CR:                              ;CARRIAGE RETURN
mov word [STATEMENT_POINTER],STATEMENT_BUF
mov di,word [BUFFER_POINTER]
mov cx,40
mov ax,''
rep stosw
jmp STATEMENT_ANALYSER
STATEMENT_SCANNER_END:

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;      STATEMENT PARSER
STATEMENT_PARSER:
._STRING:                       ;STRING
call CMP_STRING
cmp byte [TYPE],0
je ._IMM
jmp .OK
._IMM:                          ;IMM
call CMP_IMM
cmp byte [TYPE],0
je ._CONDITIONAL_DEFINITION
jmp .OK
._CONDITIONAL_DEFINITION:       ;CONDITIONAL DEFINITION
call CMP_CONDITIONAL_DEFINITION_OPCODES
cmp byte [TYPE],0
je ._DIRECTIVE
jmp .OK
._DIRECTIVE:                    ;DIRECTIVE
call CMP_DIRECITVES
cmp byte [TYPE],0
je ._OPCODE
jmp .OK
._OPCODE:                       ;OPCODE
call CMP_OPCODES
cmp byte [TYPE],0
je .REGISTERS
jmp .OK
.REGISTERS:                     ;REGISTERS
call CMP_REGISTERS
cmp byte [TYPE],0
je ._POUND
jmp .OK
._POUND:                        ;POUND
cmp word [TOKEN_BUF],0x23;#
jne ._PLUS
mov byte [TYPE],_POUND_
jmp .OK
._PLUS:                         ;PLUS
cmp word [TOKEN_BUF],0x2b;+
jne ._NINUS
mov byte [TYPE],_PLUS_
jmp .OK
._NINUS:                        ;MINUS
cmp word [TOKEN_BUF],0x2d;-
jne ._MUL
mov byte [TYPE],_MINUS_
jmp .OK
._MUL:                          ;MUL
cmp word [TOKEN_BUF],0x2a;*
jne ._UNSIGNED_DIV
mov byte [TYPE],_MUL_
jmp .OK
._UNSIGNED_DIV:                 ;UNSIGNED DIV
cmp word [TOKEN_BUF],0x2f;/
jne ._AND
mov byte [TYPE],_UNDIV_
jmp .OK
._AND:                          ;AND
cmp word [TOKEN_BUF],0x26;&
jne ._XOR
mov byte [TYPE],_AND_
jmp .OK
._XOR:                          ;XOR
cmp word [TOKEN_BUF],0x5e;^
jne ._OR
mov byte [TYPE],_XOR_
jmp .OK
._OR:                           ;OR
cmp word [TOKEN_BUF],0x7c;|
jne ._COMPLIMENT
mov byte [TYPE],_OR_
jmp .OK
._COMPLIMENT:                   ;COMPLIMENT
cmp word [TOKEN_BUF],0x7e;~
jne ._EQUAL
mov byte [TYPE],_COMPLI_
jmp .OK
._EQUAL:                        ;EQUAL
cmp word [TOKEN_BUF],0x3d;=
jne ._COMMA
mov byte [TYPE],_EQUAL_
jmp .OK
._COMMA:                        ;COMMA
cmp word [TOKEN_BUF],0x2c;,
jne ._COLON
mov byte [TYPE],_COMMA_
jmp .OK
._COLON:                        ;COLON
cmp word [TOKEN_BUF],0x3a;:
jne ._LPARA
mov byte [TYPE],_COLON_
jmp .OK
._LPARA:                        ;LPARA
cmp word [TOKEN_BUF],0x28;(
jne ._RPARA
mov byte [TYPE],_LPARA_
inc byte [LEVEL]
jmp .OK
._RPARA:                        ;RPARA
cmp word [TOKEN_BUF],0x29;(
jne ._LBRACKET
mov byte [TYPE],_RPARA_
dec byte [LEVEL]
jmp .OK
._LBRACKET:                     ;LBRACKET
cmp word [TOKEN_BUF],0x5b;[
jne ._RBRACKET
mov byte [TYPE],_LBRACK_
jmp .OK
._RBRACKET:                     ;RBRACKET
cmp word [TOKEN_BUF],0x5d;]
jne ._LBRACE
mov byte [TYPE],_RBRACK_
jmp .OK
._LBRACE:                       ;LBRACE
cmp word [TOKEN_BUF],0x7b;{
jne ._RBRACE
mov byte [TYPE],_LBRACE_
jmp .OK
._RBRACE:                       ;RBRACE
cmp word [TOKEN_BUF],0x7d;}
jne ._LSHIFT
mov byte [TYPE],_RBRACE_
jmp .OK
._LSHIFT:                       ;LSHIFT
cmp word [TOKEN_BUF],0x3c3c;<<
jne ._RSHIFT
mov byte [TYPE],_LSHIFT_
jmp .OK
._RSHIFT:                       ;RSHIFT
cmp word [TOKEN_BUF],0x3e3e;>>
jne ._SYMBOL
mov byte [TYPE],_RSHIFT_
jmp .OK
._SYMBOL:                       ;SYMBOL
mov byte [TYPE],_SYMBOL_
.OK:
mov si,TYPE
mov di,word [BUFFER_POINTER]
mov cx,40
rep movsw
add word [BUFFER_POINTER],80
jmp STATEMENT_SCANNER

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;      STATEMENT ANALYSER
STATEMENT_ANALYSER:
cmp byte [LEVEL],0
jne .ERR
mov bx,BUFFER+80
jmp NULL_STATEMENT
.ERR:
jmp ASM_ERROR.100

NULL_STATEMENT:
cmp byte [bx],_END_
jne LABEL
jmp STATEMENT_END

LABEL:
cmp byte [bx],_SYMBOL_
jne CONDITIONAL_DEFINITION
mov si,bx
add si,16
mov di,LABEL_CONTENT
mov cx,32
rep movsw
mov byte [bx],_INVALID_
call RESORTOR
call SAVE_LABEL

COLON:
cmp byte [bx],_COLON_
jne CONDITIONAL_DEFINITION
mov byte [bx],_INVALID_
call RESORTOR
cmp byte [bx],_END_
jne CONDITIONAL_DEFINITION
jmp STATEMENT_END

CONDITIONAL_DEFINITION:
cmp byte [bx],_CONDITION_
jne .TEST
.IFDEF:
cmp byte [bx+1],0
jne .IFNDEF
cmp byte [DEF_CONDITION],0
jne .ERR
inc byte [DEF_CONDITION]
add bx,80
cmp byte [bx],_SYMBOL_
jne .ERR
cmp byte [bx+80],_END_
jne .ERR
call LABEL_QUERY
setnc byte [IGNORE]
jmp STATEMENT_END
.IFNDEF:
cmp byte [bx+1],1
jne .ENDIF
cmp byte [DEF_CONDITION],0
jne .ERR
inc byte [DEF_CONDITION]
add bx,80
cmp byte [bx],_SYMBOL_
jne .ERR
cmp byte [bx+80],_END_
jne .ERR
call LABEL_QUERY
setc byte [IGNORE]
jmp STATEMENT_END
.ENDIF:
cmp byte [DEF_CONDITION],1
jne .ERR
dec byte [DEF_CONDITION]
cmp byte [bx+80],_END_
jne .ERR
mov byte [IGNORE],0
jmp STATEMENT_END
.ERR:
jmp ASM_ERROR.07
.TEST:
cmp byte [IGNORE],1
jne EXPRESSION_EVALUATION
jmp STATEMENT_END

EXPRESSION_EVALUATION:
call EXPRESSION
mov bx,BUFFER+80

DIRECTIVES:
cmp byte [bx],_DIRECITVE_
je .TIMES
jmp OPCODE
.TIMES:
cmp byte [bx+1],0
jne .BSS
mov byte [bx],_INVALID_
call RESORTOR
cmp byte [bx],_NUMBER_
jne .ERR
mov ax,word [bx+1]
mov word [TIMES_COUNT],ax
mov byte [bx],_INVALID_
call RESORTOR
jmp OPCODE
.BSS:
cmp byte [bx+1],1
jne .ORG
cmp byte [bx+80],_END_
jne .ERR
mov byte [BSS],1
jmp STATEMENT_END
.ORG:
cmp byte [bx+1],2
jne .ERR
add bx,80
cmp byte [bx],_EQUAL_
jne .ERR
add bx,80
cmp byte [bx],_NUMBER_
jne .ERR
mov ax,word [bx+1]
add word [LABEL_OFFSET],ax
mov ax,word [bx+1+2]
adc word [LABEL_OFFSET+2],ax
cmp byte [bx+80],_END_
jne .ERR
jmp STATEMENT_END
.ERR:
jmp ASM_ERROR.25

OPCODE:
cmp byte [bx],_OPCODE_
jne .ERR1
cmp byte [BSS],1
je .BSS_OPCODE
.NON_BSS_OPOCODE:
cmp word [bx+1],60
jae .ERR3
jmp _ADC
.BSS_OPCODE:
cmp word [bx+1],60
jb .ERR4
jmp _RESB
.ERR1:
jmp ASM_ERROR.20
.ERR2:
jmp ASM_ERROR.55
.ERR3:
jmp ASM_ERROR.90
.ERR4:
jmp ASM_ERROR.85

_ADC:
cmp word [bx+1],0
jne .END
mov byte [OP],0x69
call _IMMEDIATE
jc .OK
mov byte [OP],0x65
call _ZERO_PAGE
jc .OK
mov byte [OP],0x75
call _ZERO_PAGE_X
jc .OK
mov byte [OP],0x60
call _ABSOLUTE
jc .OK
mov byte [OP],0x70
call _ABSOLUTE_X
jc .OK
mov byte [OP],0x79
call _ABSOLUTE_Y
jc .OK
mov byte [OP],0x61
call _PRE_INDEXED_INDIRECT
jc .OK
mov byte [OP],0x71
call _POST_INDEXED_INDIRECT
jc .OK
jmp OPCODE.ERR2
.OK:
jmp STATEMENT_END
.END:

_AND:
cmp word [bx+1],1
jne .END
mov byte [OP],0x29
call _IMMEDIATE
jc .OK
mov byte [OP],0x25
call _ZERO_PAGE
jc .OK
mov byte [OP],0x35
call _ZERO_PAGE_X
jc .OK
mov byte [OP],0x2d
call _ABSOLUTE
jc .OK
mov byte [OP],0x3d
call _ABSOLUTE_X
jc .OK
mov byte [OP],0x39
call _ABSOLUTE_Y
jc .OK
mov byte [OP],0x21
call _PRE_INDEXED_INDIRECT
jc .OK
mov byte [OP],0x31
call _POST_INDEXED_INDIRECT
jc .OK
jmp OPCODE.ERR2
.OK:
jmp STATEMENT_END
.END:

_ASL:
cmp word [bx+1],2
jne .END
mov byte [OP],0x0a
call _ACCUMULATOR
jc .OK
mov byte [OP],0x06
call _ZERO_PAGE
jc .OK
mov byte [OP],0x16
call _ZERO_PAGE_X
jc .OK
mov byte [OP],0x0e
call _ABSOLUTE
jc .OK
mov byte [OP],0x1e
call _ABSOLUTE_X
jc .OK
jmp OPCODE.ERR2
.OK:
jmp STATEMENT_END
.END:

_BCC:
cmp word [bx+1],3
jne .END
mov byte [OP],0x90
call _RELATIVE
jc .OK
jmp OPCODE.ERR2
.OK:
jmp STATEMENT_END
.END:

_BCS:
cmp word [bx+1],4
jne .END
mov byte [OP],0xb0
call _RELATIVE
jc .OK
jmp OPCODE.ERR2
.OK:
jmp STATEMENT_END
.END:

_BEQ:
cmp word [bx+1],5
jne .END
mov byte [OP],0xf0
call _RELATIVE
jc .OK
jmp OPCODE.ERR2
.OK:
jmp STATEMENT_END
.END:

_BIT:
cmp word [bx+1],6
jne .END
mov byte [OP],0x24
call _ZERO_PAGE
jc .OK
mov byte [OP],0x2c
call _ABSOLUTE
jc .OK
jmp OPCODE.ERR2
.OK:
jmp STATEMENT_END
.END:

_BMI:
cmp word [bx+1],7
jne .END
mov byte [OP],0x30
call _RELATIVE
jc .OK
jmp OPCODE.ERR2
.OK:
jmp STATEMENT_END
.END:

_BNE:
cmp word [bx+1],8
jne .END
mov byte [OP],0xd0
call _RELATIVE
jc .OK
jmp OPCODE.ERR2
.OK:
jmp STATEMENT_END
.END:

_BPL:
cmp word [bx+1],9
jne .END
mov byte [OP],0x10
call _RELATIVE
jc .OK
jmp OPCODE.ERR2
.OK:
jmp STATEMENT_END
.END:

_BRK:
cmp word [bx+1],10
jne .END
mov byte [OP],0x00
call _IMPLIED
jc .OK
jmp OPCODE.ERR2
.OK:
jmp STATEMENT_END
.END:

_BVC:
cmp word [bx+1],11
jne .END
mov byte [OP],0x50
call _RELATIVE
jc .OK
jmp OPCODE.ERR2
.OK:
jmp STATEMENT_END
.END:

_BVS:
cmp word [bx+1],12
jne .END
mov byte [OP],0x70
call _RELATIVE
jc .OK
jmp OPCODE.ERR2
.OK:
jmp STATEMENT_END
.END:

_CLC:
cmp word [bx+1],13
jne .END
mov byte [OP],0x18
call _IMPLIED
jc .OK
jmp OPCODE.ERR2
.OK:
jmp STATEMENT_END
.END:

_CLD:
cmp word [bx+1],14
jne .END
mov byte [OP],0xd8
call _IMPLIED
jc .OK
jmp OPCODE.ERR2
.OK:
jmp STATEMENT_END
.END:

_CLI:
cmp word [bx+1],15
jne .END
mov byte [OP],0x58
call _IMPLIED
jc .OK
jmp OPCODE.ERR2
.OK:
jmp STATEMENT_END
.END:

_CLV:
cmp word [bx+1],16
jne .END
mov byte [OP],0xb8
call _IMPLIED
jc .OK
jmp OPCODE.ERR2
.OK:
jmp STATEMENT_END
.END:

_CMP:
cmp word [bx+1],17
jne .END
mov byte [OP],0xc9
call _IMMEDIATE
jc .OK
mov byte [OP],0xc5
call _ZERO_PAGE
jc .OK
mov byte [OP],0xd5
call _ZERO_PAGE_X
jc .OK
mov byte [OP],0xcd
call _ABSOLUTE
jc .OK
mov byte [OP],0xdd
call _ABSOLUTE_X
jc .OK
mov byte [OP],0xd9
call _ABSOLUTE_Y
jc .OK
mov byte [OP],0xc1
call _PRE_INDEXED_INDIRECT
jc .OK
mov byte [OP],0xd1
call _POST_INDEXED_INDIRECT
jc .OK
jmp OPCODE.ERR2
.OK:
jmp STATEMENT_END
.END:

_CPX:
cmp word [bx+1],18
jne .END
mov byte [OP],0xe0
call _IMMEDIATE
jc .OK
mov byte [OP],0xe4
call _ZERO_PAGE
jc .OK
mov byte [OP],0xec
call _ABSOLUTE
jc .OK
jmp OPCODE.ERR2
.OK:
jmp STATEMENT_END
.END:

_CPY:
cmp word [bx+1],19
jne .END
mov byte [OP],0xc0
call _IMMEDIATE
jc .OK
mov byte [OP],0xc4
call _ZERO_PAGE
jc .OK
mov byte [OP],0xcc
call _ABSOLUTE
jc .OK
jmp OPCODE.ERR2
.OK:
jmp STATEMENT_END
.END:

_DEC:
cmp word [bx+1],20
jne .END
mov byte [OP],0xc6
call _ZERO_PAGE
jc .OK
mov byte [OP],0xd6
call _ZERO_PAGE_X
jc .OK
mov byte [OP],0xce
call _ABSOLUTE
jc .OK
mov byte [OP],0xde
call _ABSOLUTE_X
jc .OK
jmp OPCODE.ERR2
.OK:
jmp STATEMENT_END
.END:

_DEX:
cmp word [bx+1],21
jne .END
mov byte [OP],0xca
call _IMPLIED
jc .OK
jmp OPCODE.ERR2
.OK:
jmp STATEMENT_END
.END:

_DEY:
cmp word [bx+1],22
jne .END
mov byte [OP],0x88
call _IMPLIED
jc .OK
jmp OPCODE.ERR2
.OK:
jmp STATEMENT_END
.END:

_EOR:
cmp word [bx+1],23
jne .END
mov byte [OP],0x49
call _IMMEDIATE
jc .OK
mov byte [OP],0x45
call _ZERO_PAGE
jc .OK
mov byte [OP],0x55
call _ZERO_PAGE_X
jc .OK
mov byte [OP],0x40
call _ABSOLUTE
jc .OK
mov byte [OP],0x50
call _ABSOLUTE_X
jc .OK
mov byte [OP],0x59
call _ABSOLUTE_Y
jc .OK
mov byte [OP],0x41
call _PRE_INDEXED_INDIRECT
jc .OK
mov byte [OP],0x51
call _POST_INDEXED_INDIRECT
jc .OK
jmp OPCODE.ERR2
.OK:
jmp STATEMENT_END
.END:

_INC:
cmp word [bx+1],24
jne .END
mov byte [OP],0xe6
call _ZERO_PAGE
jc .OK
mov byte [OP],0xf6
call _ZERO_PAGE_X
jc .OK
mov byte [OP],0xee
call _ABSOLUTE
jc .OK
mov byte [OP],0xfe
call _ABSOLUTE_X
jc .OK
jmp OPCODE.ERR2
.OK:
jmp STATEMENT_END
.END:

_INX:
cmp word [bx+1],25
jne .END
mov byte [OP],0xe8
call _IMPLIED
jc .OK
jmp OPCODE.ERR2
.OK:
jmp STATEMENT_END
.END:

_INY:
cmp word [bx+1],26
jne .END
mov byte [OP],0xc8
call _IMPLIED
jc .OK
jmp OPCODE.ERR2
.OK:
jmp STATEMENT_END
.END:

_JMP:
cmp word [bx+1],27
jne .END
mov byte [OP],0x4c
call _ABSOLUTE
jc .OK
mov byte [OP],0x6c
call _INDIRECT
jc .OK
jmp OPCODE.ERR2
.OK:
jmp STATEMENT_END
.END:

_JSR:
cmp word [bx+1],28
jne .END
mov byte [OP],0x20
call _ABSOLUTE
jc .OK
jmp OPCODE.ERR2
.OK:
jmp STATEMENT_END
.END:

_LDA:
cmp word [bx+1],29
jne .END
mov byte [OP],0xa9
call _IMMEDIATE
jc .OK
mov byte [OP],0xa5
call _ZERO_PAGE
jc .OK
mov byte [OP],0xb5
call _ZERO_PAGE_X
jc .OK
mov byte [OP],0xad
call _ABSOLUTE
jc .OK
mov byte [OP],0xbd
call _ABSOLUTE_X
jc .OK
mov byte [OP],0xb9
call _ABSOLUTE_Y
jc .OK
mov byte [OP],0xa1
call _PRE_INDEXED_INDIRECT
jc .OK
mov byte [OP],0xb1
call _POST_INDEXED_INDIRECT
jc .OK
jmp OPCODE.ERR2
.OK:
jmp STATEMENT_END
.END:

_LDX:
cmp word [bx+1],30
jne .END
mov byte [OP],0xa2
call _IMMEDIATE
jc .OK
mov byte [OP],0xa6
call _ZERO_PAGE
jc .OK
mov byte [OP],0xb6
call _ZERO_PAGE_Y
jc .OK
mov byte [OP],0xae
call _ABSOLUTE
jc .OK
mov byte [OP],0xbe
call _ABSOLUTE_Y
jc .OK
jmp OPCODE.ERR2
.OK:
jmp STATEMENT_END
.END:

_LDY:
cmp word [bx+1],31
jne .END
mov byte [OP],0xa0
call _IMMEDIATE
jc .OK
mov byte [OP],0xa4
call _ZERO_PAGE
jc .OK
mov byte [OP],0xb4
call _ZERO_PAGE_X
jc .OK
mov byte [OP],0xac
call _ABSOLUTE
jc .OK
mov byte [OP],0xbc
call _ABSOLUTE_X
jc .OK
jmp OPCODE.ERR2
.OK:
jmp STATEMENT_END
.END:

_LSR:
cmp word [bx+1],32
jne .END
mov byte [OP],0x4a
call _ACCUMULATOR
jc .OK
mov byte [OP],0x46
call _ZERO_PAGE
jc .OK
mov byte [OP],0x56
call _ZERO_PAGE_X
jc .OK
mov byte [OP],0x4e
call _ABSOLUTE
jc .OK
mov byte [OP],0x5e
call _ABSOLUTE_X
jc .OK
jmp OPCODE.ERR2
.OK:
jmp STATEMENT_END
.END:

_NOP:
cmp word [bx+1],33
jne .END
mov byte [OP],0xea
call _IMPLIED
jc .OK
jmp OPCODE.ERR2
.OK:
jmp STATEMENT_END
.END:

_ORA:
cmp word [bx+1],34
jne .END
mov byte [OP],0x09
call _IMMEDIATE
jc .OK
mov byte [OP],0x05
call _ZERO_PAGE
jc .OK
mov byte [OP],0x15
call _ZERO_PAGE_X
jc .OK
mov byte [OP],0x0d
call _ABSOLUTE
jc .OK
mov byte [OP],0x10
call _ABSOLUTE_X
jc .OK
mov byte [OP],0x19
call _ABSOLUTE_Y
jc .OK
mov byte [OP],0x01
call _PRE_INDEXED_INDIRECT
jc .OK
mov byte [OP],0x11
call _POST_INDEXED_INDIRECT
jc .OK
jmp OPCODE.ERR2
.OK:
jmp STATEMENT_END
.END:

_PHA:
cmp word [bx+1],35
jne .END
mov byte [OP],0x48
call _IMPLIED
jc .OK
jmp OPCODE.ERR2
.OK:
jmp STATEMENT_END
.END:

_PHP:
cmp word [bx+1],36
jne .END
mov byte [OP],0x08
call _IMPLIED
jc .OK
jmp OPCODE.ERR2
.OK:
jmp STATEMENT_END
.END:

_PLA:
cmp word [bx+1],37
jne .END
mov byte [OP],0x68
call _IMPLIED
jc .OK
jmp OPCODE.ERR2
.OK:
jmp STATEMENT_END
.END:

_PLP:
cmp word [bx+1],38
jne .END
mov byte [OP],0x28
call _IMPLIED
jc .OK
jmp OPCODE.ERR2
.OK:
jmp STATEMENT_END
.END:

_ROL:
cmp word [bx+1],39
jne .END
mov byte [OP],0x2a
call _ACCUMULATOR
jc .OK
mov byte [OP],0x26
call _ZERO_PAGE
jc .OK
mov byte [OP],0x36
call _ZERO_PAGE_X
jc .OK
mov byte [OP],0x2e
call _ABSOLUTE
jc .OK
mov byte [OP],0x3e
call _ABSOLUTE_X
jc .OK
jmp OPCODE.ERR2
.OK:
jmp STATEMENT_END
.END:

_ROR:
cmp word [bx+1],40
jne .END
mov byte [OP],0x6a
call _ACCUMULATOR
jc .OK
mov byte [OP],0x66
call _ZERO_PAGE
jc .OK
mov byte [OP],0x76
call _ZERO_PAGE_X
jc .OK
mov byte [OP],0x6e
call _ABSOLUTE
jc .OK
mov byte [OP],0x7e
call _ABSOLUTE_X
jc .OK
jmp OPCODE.ERR2
.OK:
jmp STATEMENT_END
.END:

_RTI:
cmp word [bx+1],41
jne .END
mov byte [OP],0x4d
call _IMPLIED
jc .OK
jmp OPCODE.ERR2
.OK:
jmp STATEMENT_END
.END:

_RTS:
cmp word [bx+1],42
jne .END
mov byte [OP],0x60
call _IMPLIED
jc .OK
jmp OPCODE.ERR2
.OK:
jmp STATEMENT_END
.END:

_SBC:
cmp word [bx+1],43
jne .END
mov byte [OP],0xe9
call _IMMEDIATE
jc .OK
mov byte [OP],0xe5
call _ZERO_PAGE
jc .OK
mov byte [OP],0xf5
call _ZERO_PAGE_X
jc .OK
mov byte [OP],0xed
call _ABSOLUTE
jc .OK
mov byte [OP],0xfd
call _ABSOLUTE_X
jc .OK
mov byte [OP],0xf9
call _ABSOLUTE_Y
jc .OK
mov byte [OP],0xe1
call _PRE_INDEXED_INDIRECT
jc .OK
mov byte [OP],0xf1
call _POST_INDEXED_INDIRECT
jc .OK
jmp OPCODE.ERR2
.OK:
jmp STATEMENT_END
.END:

_SEC:
cmp word [bx+1],44
jne .END
mov byte [OP],0x38
call _IMPLIED
jc .OK
jmp OPCODE.ERR2
.OK:
jmp STATEMENT_END
.END:

_SED:
cmp word [bx+1],45
jne .END
mov byte [OP],0xf8
call _IMPLIED
jc .OK
jmp OPCODE.ERR2
.OK:
jmp STATEMENT_END
.END:

_SEI:
cmp word [bx+1],46
jne .END
mov byte [OP],0x78
call _IMPLIED
jc .OK
jmp OPCODE.ERR2
.OK:
jmp STATEMENT_END
.END:

_STA:
cmp word [bx+1],47
jne .END
mov byte [OP],0x85
call _ZERO_PAGE
jc .OK
mov byte [OP],0x95
call _ZERO_PAGE_X
jc .OK
mov byte [OP],0x80
call _ABSOLUTE
jc .OK
mov byte [OP],0x90
call _ABSOLUTE_X
jc .OK
mov byte [OP],0x99
call _ABSOLUTE_Y
jc .OK
mov byte [OP],0x81
call _PRE_INDEXED_INDIRECT
jc .OK
mov byte [OP],0x91
call _POST_INDEXED_INDIRECT
jc .OK
jmp OPCODE.ERR2
.OK:
jmp STATEMENT_END
.END:

_STX:
cmp word [bx+1],48
jne .END
mov byte [OP],0x86
call _ZERO_PAGE
jc .OK
mov byte [OP],0x96
call _ZERO_PAGE_Y
jc .OK
mov byte [OP],0x8e
call _ABSOLUTE
jc .OK
jmp OPCODE.ERR2
.OK:
jmp STATEMENT_END
.END:

_STY:
cmp word [bx+1],49
jne .END
mov byte [OP],0x84
call _ZERO_PAGE
jc .OK
mov byte [OP],0x94
call _ZERO_PAGE_X
jc .OK
mov byte [OP],0x8c
call _ABSOLUTE
jc .OK
jmp OPCODE.ERR2
.OK:
jmp STATEMENT_END
.END:

_TAX:
cmp word [bx+1],50
jne .END
mov byte [OP],0xaa
call _IMPLIED
jc .OK
jmp OPCODE.ERR2
.OK:
jmp STATEMENT_END
.END:

_TAY:
cmp word [bx+1],51
jne .END
mov byte [OP],0xa8
call _IMPLIED
jc .OK
jmp OPCODE.ERR2
.OK:
jmp STATEMENT_END
.END:

_TSX:
cmp word [bx+1],52
jne .END
mov byte [OP],0xba
call _IMPLIED
jc .OK
jmp OPCODE.ERR2
.OK:
jmp STATEMENT_END
.END:

_TXA:
cmp word [bx+1],53
jne .END
mov byte [OP],0x8a
call _IMPLIED
jc .OK
jmp OPCODE.ERR2
.OK:
jmp STATEMENT_END
.END:

_TXS:
cmp word [bx+1],54
jne .END
mov byte [OP],0x9a
call _IMPLIED
jc .OK
jmp OPCODE.ERR2
.OK:
jmp STATEMENT_END
.END:

_TYA:
cmp word [bx+1],55
jne .END
mov byte [OP],0x98
call _IMPLIED
jc .OK
jmp OPCODE.ERR2
.OK:
jmp STATEMENT_END
.END:

_EQU:
cmp word [bx+1],56
jne .END
cmp byte [LABEL_CONTENT],0
je .ERR
add bx,80
cmp byte [bx],_NUMBER_
je .OK
cmp byte [bx],_STRING_
je .OK
jmp .ERR
.OK:
cmp byte [bx+80],_END_
jne .ERR
call REPLACE_LABEL
jmp STATEMENT_END
.ERR:
jmp ASM_ERROR.55
.END:

_DB:
cmp word [bx+1],57
jne .END
mov word [BUFFER_TMP_POINTER],BUFFER_TMP
.LOOP:
add bx,80
cmp byte [bx],_NUMBER_
je .NUMBER
cmp byte [bx],_STRING_
je .STRING
jmp .ERR1
.MARK:
add bx,80
cmp byte [bx],_END_
je .SAVE
cmp byte [bx],_COMMA_
jne .ERR2
jmp .LOOP
.NUMBER:
call DB_NUMBER
jmp .MARK
.STRING:
call DB_STRING
jmp .MARK
.ERR1:
jmp ASM_ERROR.55
.ERR2:
jmp ASM_ERROR.60
.SAVE:
call DB_DW_DD_SAVE
.END:

_DW:
cmp word [bx+1],58
jne .END
mov word [BUFFER_TMP_POINTER],BUFFER_TMP
.LOOP:
add bx,80
cmp byte [bx],_NUMBER_
je .NUMBER
cmp byte [bx],_STRING_
je .STRING
jmp .ERR1
.MARK:
add bx,80
cmp byte [bx],_END_
je .SAVE
cmp byte [bx],_COMMA_
jne .ERR2
jmp .LOOP
.NUMBER:
call DW_NUMBER
jmp .MARK
.STRING:
call DW_STRING
jmp .MARK
.ERR1:
jmp ASM_ERROR.55
.ERR2:
jmp ASM_ERROR.60
.SAVE:
call DB_DW_DD_SAVE
.END:

_DD:
cmp word [bx+1],59
jne .END
mov word [BUFFER_TMP_POINTER],BUFFER_TMP
.LOOP:
add bx,80
cmp byte [bx],_NUMBER_
je .NUMBER
cmp byte [bx],_STRING_
je .STRING
jmp .ERR1
.MARK:
add bx,80
cmp byte [bx],_END_
je .SAVE
cmp byte [bx],_COMMA_
jne .ERR2
jmp .LOOP
.NUMBER:
call DD_NUMBER
jmp .MARK
.STRING:
call DD_STRING
jmp .MARK
.ERR1:
jmp ASM_ERROR.55
.ERR2:
jmp ASM_ERROR.60
.SAVE:
call DB_DW_DD_SAVE
.END:

_RESB:
cmp word [bx+1],60
jne .END
add bx,80
cmp byte [bx],_NUMBER_
jne .ERR
cmp byte [bx+80],_END_
jne .ERR
.LOOP1:
cmp word [TIMES_COUNT],0
je .END
dec word [TIMES_COUNT]
mov cx,word [bx+1]
.LOOP2:
cmp cx,0
je .LOOP1
add word [LABEL_OFFSET],1
adc word [LABEL_OFFSET+2],0
dec cx
jmp .LOOP2
.ERR:
jmp ASM_ERROR.55
.END:

_RESW:
cmp word [bx+1],61
jne .END
add bx,80
cmp byte [bx],_NUMBER_
jne .ERR
cmp byte [bx+80],_END_
jne .ERR
.LOOP1:
cmp word [TIMES_COUNT],0
je .END
dec word [TIMES_COUNT]
mov cx,word [bx+1]
.LOOP2:
cmp cx,0
je .LOOP1
add word [LABEL_OFFSET],2
adc word [LABEL_OFFSET+2],0
dec cx
jmp .LOOP2
.ERR:
jmp ASM_ERROR.55
.END:

_RESD:
cmp word [bx+1],62
jne .END
add bx,80
cmp byte [bx],_NUMBER_
jne .ERR
cmp byte [bx+80],_END_
jne .ERR
.LOOP1:
cmp word [TIMES_COUNT],0
je .END
dec word [TIMES_COUNT]
mov cx,word [bx+1]
.LOOP2:
cmp cx,0
je .LOOP1
add word [LABEL_OFFSET],4
adc word [LABEL_OFFSET+2],0
dec cx
jmp .LOOP2
.ERR:
jmp ASM_ERROR.55
.END:

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;      STATEMENT END
STATEMENT_END:
cmp byte [PASS],'1'
je .PASS1
jmp .PASS2
.PASS1:
cmp word [SOURCE_FILE_SIZE+2],0
jne .PASS1_NOT_END
cmp word [SOURCE_FILE_SIZE],0
jne .PASS1_NOT_END
cmp byte [DEF_CONDITION],0
jne .ERR
mov byte [PASS],'2'
mov byte [BSS],0
mov byte [LABEL_GLOBAL],0
mov di,LINE_NO
mov cx,4
mov ax,0
rep stosw
mov word [SOURCE_FILE_POS],0
mov word [SOURCE_FILE_POS+2],cs
add word [SOURCE_FILE_POS+2],0x1000
mov ax,0x4202
mov bx,word [SOURCE_FILE_HANDLE]
mov cx,0
mov dx,0
int 0x21
mov word [SOURCE_FILE_SIZE],ax
mov word [SOURCE_FILE_SIZE+2],dx
mov word [LABEL_OFFSET],0
mov word [LABEL_OFFSET+2],0
.PASS1_NOT_END:
call INC_LINE_NO
jmp STATEMENT_FETCHER
.ERR:
jmp ASM_ERROR.05
.PASS2:
cmp word [SOURCE_FILE_SIZE+2],0
jne .PASS2_NOT_END
cmp word [SOURCE_FILE_SIZE],0
jne .PASS2_NOT_END
jmp ASM_QUIT
.PASS2_NOT_END:
call INC_LINE_NO
jmp STATEMENT_FETCHER

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;      ASM ERROR
ASM_ERROR:
.105:
mov bp,MESSAGE.105
call ERROR_PROC4
jmp .END
.100:
mov bp,MESSAGE.100
call ERROR_PROC4
jmp .END
.95:
mov bp,MESSAGE.95
call ERROR_PROC4
jmp .END
.90:
mov bp,MESSAGE.90
call ERROR_PROC4
jmp .END
.85:
mov bp,MESSAGE.85
call ERROR_PROC4
jmp .END
.80:
mov bp,MESSAGE.80
call ERROR_PROC4
jmp .END
.75:
mov bp,MESSAGE.75
call ERROR_PROC3
jmp .END
.70:
mov bp,MESSAGE.70
call ERROR_PROC3
jmp .END
.65:
mov bp,MESSAGE.65
call ERROR_PROC3
jmp .END
.60:
mov bp,MESSAGE.60
call ERROR_PROC4
jmp .END
.50:
mov bp,MESSAGE.50
call ERROR_PROC2
jmp .END
.55:
mov bp,MESSAGE.55
call ERROR_PROC4
jmp .END
.45:
mov bp,MESSAGE.45
call ERROR_PROC4
jmp .END
.40:
mov bp,MESSAGE.40
call ERROR_PROC4
jmp .END
.35:
mov bp,MESSAGE.35
call ERROR_PROC1
jmp .END
.30:
mov bp,MESSAGE.30
call ERROR_PROC1
jmp .END
.25:
mov bp,MESSAGE.25
call ERROR_PROC4
jmp .END
.20:
mov bp,MESSAGE.20
call ERROR_PROC4
jmp .END
.15:
mov bp,MESSAGE.15
call ERROR_PROC4
jmp .END
.10:
mov bp,MESSAGE.10
call ERROR_PROC4
jmp .END
.07:
mov bp,MESSAGE.07
call ERROR_PROC4
jmp .END
.06:
mov bp,MESSAGE.06
call ERROR_PROC0
jmp .END
.05:
mov bp,MESSAGE.05
call ERROR_PROC0
jmp .END
.04:
mov bp,MESSAGE.04
call ERROR_PROC0
jmp .END
.03:
mov bp,MESSAGE.03
call ERROR_PROC0
jmp .EXIT
.02:
mov bp,MESSAGE.02
call ERROR_PROC0
jmp .EXIT
.01:
mov bp,MESSAGE.01
call ERROR_PROC0
jmp .QUIT
.00:
mov bp,MESSAGE.00
call ERROR_PROC0
jmp .QUIT

.END:
call CLOSE_BINARY_FILE
call DELETE_BINARY_FILE
.EXIT:
call CLOSE_SOURCE_FILE
.QUIT:
jmp ASM_QUIT

ERROR_PROC0:
mov si,bp
call DIS_PROC
retn

ERROR_PROC1:
mov si,LINE_NO
mov di,BUFFER_TMP
mov cx,8
.L1:
cmp byte [si],0
jne .L2
inc si
loop .L1
.L2:
lodsb
add al,0x30
stosb
loop .L2
mov ax,': '
stosw
mov si,TOKEN_BUF
.L3:
lodsb
cmp al,0
je .L4
stosb
jmp .L3
.L4:
mov ax,', '
stosw
mov si,bp
.L5:
lodsb
stosb
cmp al,0
jne .L5
mov si,BUFFER_TMP
call DIS_PROC
retn

ERROR_PROC2:
mov si,LINE_NO
mov di,BUFFER_TMP
mov cx,8
.L1:
cmp byte [si],0
jne .L2
inc si
loop .L1
.L2:
lodsb
add al,0x30
stosb
loop .L2
mov ax,': '
stosw
mov si,bx
add si,16
.L3:
lodsb
cmp al,0
je .L4
stosb
jmp .L3
.L4:
mov ax,', '
stosw
mov si,bp
.L5:
lodsb
stosb
cmp al,0
jne .L5
mov si,BUFFER_TMP
call DIS_PROC
retn

ERROR_PROC3:
mov si,LINE_NO
mov di,BUFFER_TMP
mov cx,8
.L1:
cmp byte [si],0
jne .L2
inc si
loop .L1
.L2:
lodsb
add al,0x30
stosb
loop .L2
mov ax,': '
stosw
mov si,LABEL_BACKUP
.L3:
lodsb
cmp al,0
je .L4
stosb
jmp .L3
.L4:
mov ax,', '
stosw
mov si,bp
.L5:
lodsb
stosb
cmp al,0
jne .L5
mov si,BUFFER_TMP
call DIS_PROC
retn

ERROR_PROC4:
mov si,LINE_NO
mov di,BUFFER_TMP
mov cx,8
.L1:
cmp byte [si],0
jne .L2
inc si
loop .L1
.L2:
lodsb
add al,0x30
stosb
loop .L2
mov ax,': '
stosw
mov si,bp
.L4:
lodsb
stosb
cmp al,0
jne .L4
mov si,BUFFER_TMP
call DIS_PROC
retn

DIS_PROC:
.LOOP:
lodsb
cmp al,0
je .END
mov ah,0xe
mov bh,0
int 0x10
jmp .LOOP
.END:
retn

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;      ASM QUIT
ASM_QUIT:
mov ax,0x4c00
int 0x21

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;      ROUTINE;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
UPDATE_STATEMENT_POINTER:
mov word [STATEMENT_POINTER],si
retn

INC_LINE_NO:
mov si,LINE_NO+7
.LOOP:
mov al,byte [si]
add al,1
aaa
mov byte [si],al
jnc .END
dec si
jmp .LOOP
.END:
retn

RESET_TYPE:
mov byte [TYPE],0
retn
RESET_VALUE:
mov di,VALUE
mov cx,14
mov al,0
rep stosb
retn
RESET_TOKEN_BUF:
mov di,TOKEN_BUF
mov cx,32
mov ax,0
rep stosw
retn
RESET_BUFFER_TMP:
mov di,BUFFER_TMP
mov cx,32
mov ax,0
rep stosw
retn

LOWER_CASE:
mov si,BUFFER_TMP
.LOOP:
cmp byte [si],0
je .END
cmp byte [si],0x5a
ja .OK
cmp byte [si],0x41
jb .OK
add byte [si],0x20
.OK:
inc si
jmp .LOOP
.END:
retn
TOKEN_TO_TMP:
mov cx,32
mov si,TOKEN_BUF
mov di,BUFFER_TMP
rep movsw
retn
TMP_TO_TOKEN:
mov cx,32
mov si,BUFFER_TMP
mov di,TOKEN_BUF
rep movsw
retn
LOCATE_ENDING:
mov ax,si
.LOOP:
inc si
cmp byte [si],0
jne .LOOP
mov cx,si
sub cx,ax
retn

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;CMP STRING
CMP_STRING:
cmp byte [TOKEN_BUF],'"'
je .OK1
cmp byte [TOKEN_BUF],"'"
je .OK1
jmp .END
.OK1:
call RESET_BUFFER_TMP
mov si,TOKEN_BUF
mov di,BUFFER_TMP
lodsb
mov ah,al
.LOOP:
lodsb
cmp al,ah
je .OK2
stosb
jmp .LOOP
.OK2:
call TMP_TO_TOKEN
mov ax,word [BUFFER_TMP]
mov word [VALUE],ax
mov ax,word [BUFFER_TMP+2]
mov word [VALUE+2],ax
mov byte [TYPE],_STRING_
.END:
retn

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;CMP IMM
CMP_IMM:
call TOKEN_TO_TMP
call LOWER_CASE

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;CMP HEXADECIMAL
CMP_HEXADECIMAL:
mov si,BUFFER_TMP
cmp byte [BUFFER_TMP],'$'
je .FLAG1
cmp word [BUFFER_TMP],'0x'
je .FLAG2
jmp .FLAG3
.FLAG1:
cmp byte [BUFFER_TMP+1],0
jne .$_MARK
jmp .ERR
.$_MARK:
mov byte [BUFFER_TMP],'0'
jmp .DIGIT_CHECK
.FLAG2:
cmp byte [BUFFER_TMP+2],0
jne .0x_MARK
jmp .ERR
.0x_MARK:
mov word [BUFFER_TMP],'00'
jmp .DIGIT_CHECK
.FLAG3:
cmp byte [si],'0'
jb .DISMATCH
cmp byte [si],'9'
ja .DISMATCH
call LOCATE_ENDING
cmp byte [si-1],'h'
je .H_MARK
.DISMATCH:
jmp .QUIT
.H_MARK:
mov byte [si-1],0
mov si,BUFFER_TMP
.DIGIT_CHECK:
cmp byte [si],'0'
jb .A_F
cmp byte [si],'9'
ja .A_F
jmp .DIGIT_CHECK_OK
.A_F:
cmp byte [si],'a'
jb .ERR
cmp byte [si],'f'
ja .ERR
sub byte [si],0x27
.DIGIT_CHECK_OK:
cmp byte [si+1],0
je .ENCODING
inc si
jmp .DIGIT_CHECK
.ENCODING:
mov si,BUFFER_TMP
mov bp,16
call NUMBER_ENCODING
mov word [VALUE],ax
mov word [VALUE+2],dx
.END:
mov byte [TYPE],_NUMBER_
retn
.ERR:
jmp ASM_ERROR.30
.QUIT:

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;CMP OCTAL
CMP_OCTAL:
mov si,BUFFER_TMP
cmp byte [si],'0'
jb .DISMATCH
cmp byte [si],'7'
ja .DISMATCH
call LOCATE_ENDING
cmp byte [si-1],'o'
je .O_MARK
cmp byte [si-1],'q'
je .Q_MARK
.DISMATCH:
jmp .QUIT
.O_MARK:
.Q_MARK:
mov byte [si-1],0
mov si,BUFFER_TMP
.DIGIT_CHECK:
cmp byte [si],'0'
jb .ERR
cmp byte [si],'7'
ja .ERR
cmp byte [si+1],0
je .ENCODING
inc si
jmp .DIGIT_CHECK
.ENCODING:
mov si,BUFFER_TMP
mov bp,8
call NUMBER_ENCODING
mov word [VALUE],ax
mov word [VALUE+2],dx
.END:
mov byte [TYPE],_NUMBER_
retn
.ERR:
jmp ASM_ERROR.30
.QUIT:

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;CMP BINARY
CMP_BINARY:
mov si,BUFFER_TMP
cmp byte [si],'0'
jb .DISMATCH
cmp byte [si],'1'
ja .DISMATCH
call LOCATE_ENDING
cmp byte [si-1],'b'
je .B_MARK
.DISMATCH:
jmp .QUIT
.B_MARK:
mov byte [si-1],0
mov si,BUFFER_TMP
.DIGIT_CHECK:
cmp byte [si],'0'
jb .ERR
cmp byte [si],'1'
ja .ERR
cmp byte [si+1],0
je .ENCODING
inc si
jmp .DIGIT_CHECK
.ENCODING:
mov si,BUFFER_TMP
mov bp,2
call NUMBER_ENCODING
mov word [VALUE],ax
mov word [VALUE+2],dx
.END:
mov byte [TYPE],_NUMBER_
retn
.ERR:
jmp ASM_ERROR.30
.QUIT:

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;CMP DECIMAL
CMP_DECIMAL:
mov si,BUFFER_TMP
cmp byte [si],'0'
jb .DISMATCH
cmp byte [si],'9'
ja .DISMATCH
call LOCATE_ENDING
cmp byte [si-1],'d'
je .D_MARK
jmp .INIT
.DISMATCH:
jmp .QUIT
.D_MARK:
mov byte [si-1],0
.INIT:
mov si,BUFFER_TMP
.DIGIT_CHECK:
cmp byte [si],'0'
jb .ERR
cmp byte [si],'9'
ja .ERR
cmp byte [si+1],0
je .ENCODING
inc si
jmp .DIGIT_CHECK
.ENCODING:
mov si,BUFFER_TMP
mov bp,10
call NUMBER_ENCODING
mov word [VALUE],ax
mov word [VALUE+2],dx
.END:
mov byte [TYPE],_NUMBER_
retn
.ERR:
jmp ASM_ERROR.30
.QUIT:

retn

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;NUMBER ENCODING
NUMBER_ENCODING:
mov bx,0
mov cx,0
.LOOP:
mov ax,bx
mul bp
mov bx,ax
push dx
mov ax,cx
mul bp
cmp dx,0
jne .ERR
pop cx
add ax,cx
jc .ERR
mov cx,ax
lodsb
sub al,0x30
cbw
add bx,ax
adc cx,0
jc .ERR
cmp byte [si],0
je .END
jmp .LOOP
.END:
mov ax,bx
mov dx,cx
retn
.ERR:
jmp ASM_ERROR.35

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;CMP CONDITIONAL DEFINITION_OPCODES
CMP_CONDITIONAL_DEFINITION_OPCODES:
mov dx,KEYWORD.CONDITIONAL_DEFINITION_OPCODES
call CMP_PROC
jnc .END
mov byte [TYPE],_CONDITION_
.END:
retn
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;CMP DIRECTIVES
CMP_DIRECITVES:
mov dx,KEYWORD.DIRECTIVES
call CMP_PROC
jnc .END
mov byte [TYPE],_DIRECITVE_
.END:
retn
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;CMP OPCODES
CMP_OPCODES:
mov dx,KEYWORD.OPCODES
call CMP_PROC
jnc .END
mov byte [TYPE],_OPCODE_
.END:
retn
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;CMP REGISTERS
CMP_REGISTERS:
mov dx,KEYWORD.REGISTERS
call CMP_PROC
jnc .END
.A_REGISTER:
cmp byte [VALUE],0
jne .X_REGISTER
mov byte [TYPE],_A_REG_
jmp .END
.X_REGISTER:
cmp byte [VALUE],1
jne .Y_REGISTER
mov byte [TYPE],_X_REG_
jmp .END
.Y_REGISTER:
mov byte [TYPE],_Y_REG_
.END:
retn

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;CMP PROCEDURE
CMP_PROC:
call TOKEN_TO_TMP
call LOWER_CASE
mov si,BUFFER_TMP
call LOCATE_ENDING
mov bx,cx
mov bp,0
.LOOP:
mov si,dx
lodsb
cbw
cmp ax,bx
jne .DISMATCH
mov di,BUFFER_TMP
mov cx,bx
rep cmpsb
je .FOUND
.DISMATCH:
mov si,dx
add si,ax
cmp byte [si+1],0
je .NOT_FOUND
add dx,ax
inc dx
inc bp
jmp .LOOP
.FOUND:
mov word [VALUE],bp
stc
retn
.NOT_FOUND:
clc
retn

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;       EXPRESSION
EXPRESSION:
call COMPLI_PLUS_RESOLUTION
call COMPLI_MINUS_RESOLUTION
call SYMBOL_RESOLUTION
call RIP_PARENTHESE
call TEST_LEVEL
call DETERMINE_TOP_LEVEL
call RESORTOR
.LOOP:
call UNARY
call BINARY_MUL
call BINARY_UNDIV
call BINARY_PLUS_MINUS
call BINARY_LSHIFT
call BINARY_RSHIFT
call BINARY_AND
call BINARY_XOR
call BINARY_OR
call ORPHAN_NUMBER
cmp byte [LEVEL],0
je .END
dec byte [LEVEL]
jmp .LOOP
.END:
retn

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;       COMPLI PLUS RESOLUTION
COMPLI_PLUS_RESOLUTION:
mov bx,BUFFER+80
.LOOP:
cmp byte [bx],_END_
je .END
cmp byte [bx],_COMPLI_
je .OK
.SKIP:
add bx,80
jmp .LOOP
.OK:
cmp byte [bx+80],_PLUS_
jne .SKIP
mov byte [bx],_COMPLI_PLUS_
mov byte [bx+80],_INVALID_
jmp .LOOP
.END:
retn

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;       COMPLI MINUS RESOLUTION
COMPLI_MINUS_RESOLUTION:
mov bx,BUFFER+80
.LOOP:
cmp byte [bx],_END_
je .END
cmp byte [bx],_COMPLI_
je .OK
.SKIP:
add bx,80
jmp .LOOP
.OK:
cmp byte [bx+80],_MINUS_
jne .SKIP
mov byte [bx],_COMPLI_MINUS_
mov byte [bx+80],_INVALID_
jmp .LOOP
.END:
retn

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;       SYMBOL RESOLUTION
SYMBOL_RESOLUTION:
mov bx,BUFFER+80
.LOOP:
cmp byte [bx],_END_
je .END
cmp byte [bx],_SYMBOL_
je .OK
.SKIP:
add bx,80
jmp .LOOP
.OK:
call LOAD_LABEL
mov byte [bx],_NUMBER_
jmp .LOOP
.END:
retn

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;       RIP PARENTHESE
RIP_PARENTHESE:
mov bx,BUFFER+80
.LOOP:
cmp byte [bx],_END_
je .END
cmp byte [bx],_LPARA_
je .LPARA
cmp byte [bx],_RPARA_
je .LPARA
.SKIP:
add bx,80
jmp .LOOP
.LPARA:
.RPARA:
mov byte [bx],_INVALID_
jmp .SKIP
.END:
retn

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;       TEST LEVEL
TEST_LEVEL:
mov bx,BUFFER+80
.LOOP:
cmp byte [bx],_END_
je .END
cmp byte [bx],_NUMBER_
je .OK
cmp byte [bx],_STRING_
je .OK
cmp byte [bx],_PLUS_
je .OK
cmp byte [bx],_MINUS_
je .OK
cmp byte [bx],_MUL_
je .OK
cmp byte [bx],_UNDIV_
je .OK
cmp byte [bx],_LSHIFT_
je .OK
cmp byte [bx],_RSHIFT_
je .OK
cmp byte [bx],_COMPLI_
je .OK
cmp byte [bx],_COMPLI_PLUS_
je .OK
cmp byte [bx],_COMPLI_MINUS_
je .OK
cmp byte [bx],_AND_
je .OK
cmp byte [bx],_XOR_
je .OK
cmp byte [bx],_OR_
je .OK
cmp byte [bx],_INVALID_
je .OK
cmp byte [bx+15],0
jne .ERR
.OK:
add bx,80
jmp .LOOP
.END:
retn
.ERR:
jmp ASM_ERROR.105

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;       DETERMINE TOP LEVEL
DETERMINE_TOP_LEVEL:
mov bx,BUFFER+80+15
mov al,0
.LOOP:
cmp byte [bx],_END_
je .END
cmp byte [bx],al
jbe .SKIP
inc al
.SKIP:
add bx,80
jmp .LOOP
.END:
mov byte [LEVEL],al
retn

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;       UNARY ~ + -
UNARY:
.INIT:
mov bx,BUFFER+80
.LOOP:
cmp byte [bx],_END_
je .END
cmp byte [bx],_COMPLI_
je .OK
cmp byte [bx],_PLUS_
je .OK
cmp byte [bx],_MINUS_
je .OK
cmp byte [bx],_COMPLI_PLUS_
je .OK
cmp byte [bx],_COMPLI_MINUS_
je .OK
.SKIP:
add bx,80
jmp .LOOP
.END:
retn
.OK:
mov al,byte [LEVEL]
cmp byte [bx+15],al
jne .SKIP
cmp byte [bx+80],_NUMBER_
jne .SKIP
cmp byte [bx+80+15],al
jne .SKIP
cmp byte [bx-80],_NUMBER_
je .SKIP
.COMPLI:
cmp byte [bx],_COMPLI_
jne .PLUS
not word [bx+80+1]
not word [bx+80+1+2]
jmp .DEL
.PLUS:
cmp byte [bx],_PLUS_
jne .MINUS
jmp .DEL
.MINUS:
cmp byte [bx],_MINUS_
jne .COMPLI_PLUS
neg word [bx+80+1]
adc word [bx+80+1+2],0
neg word [bx+80+1+2]
jmp .DEL
.COMPLI_PLUS:
cmp byte [bx],_COMPLI_PLUS_
jne .COMPLI_MINUS
not word [bx+80+1]
not word [bx+80+1+2]
jmp .DEL
.COMPLI_MINUS:
neg word [bx+80+1]
adc word [bx+80+1+2],0
neg word [bx+80+1+2]
not word [bx+80+1]
not word [bx+80+1+2]
jmp .DEL
.DEL:
mov byte [bx],_INVALID_
call RESORTOR
jmp .INIT

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;       BINARY *
BINARY_MUL:
mov bx,BUFFER+80
.LOOP:
cmp byte [bx],_END_
je .END
cmp byte [bx],_MUL_
je .OK
.SKIP:
add bx,80
jmp .LOOP
.OK:
mov al,byte [LEVEL]
cmp byte [bx+15],al
jne .SKIP
mov al,byte [bx-80+15]
mov ah,byte [bx+80+15]
cmp al,ah
jne .SKIP
mov al,byte [bx-80]
mov ah,byte [bx+80]
cmp ax,_NUMBER_+_NUMBER_*256
jne .SKIP
mov ax,word [bx+80+1]
mul word [bx-80+1]
mov word [bx+1],ax
mov word [bx+1+2],dx
mov ax,word [bx+80+1]
mul word [bx-80+1+2]
add word [bx+1+2],ax
adc word [bx+1+4],dx
mov ax,word [bx+80+1+2]
mul word [bx-80+1]
add word [bx+1+2],ax
adc word [bx+1+4],dx
mov ax,word [bx+80+1+2]
mul word [bx-80+1+2]
add word [bx+1+4],ax
adc word [bx+1+6],dx
.DEL:
mov byte [bx],_NUMBER_
mov byte [bx+80],_INVALID_
mov byte [bx-80],_INVALID_
call RESORTOR
jmp .LOOP
.END:
retn

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;       BINARY /
BINARY_UNDIV:
mov bx,BUFFER+80
.LOOP:
cmp byte [bx],_END_
je .END
cmp byte [bx],_UNDIV_
je .OK
.SKIP:
add bx,80
jmp .LOOP
.OK:
mov al,byte [LEVEL]
cmp byte [bx+15],al
jne .SKIP
mov al,byte [bx-80+15]
mov ah,byte [bx+80+15]
cmp al,ah
jne .SKIP
mov al,byte [bx-80]
mov ah,byte [bx+80]
cmp ax,_NUMBER_+_NUMBER_*256
jne .SKIP
call UNDIV_PROC
.DEL:
mov byte [bx],_NUMBER_
mov byte [bx-80],_INVALID_
mov byte [bx+80],_INVALID_
call RESORTOR
jmp .LOOP
.END:
retn

UNDIV_PROC:
cmp word [bx+80+1+2],0
jne .PROC2
cmp word [bx+80+1],0
je .ERR
.PROC1:
mov dx,0
mov ax,word [bx-80+1+2]
div word [bx+80+1]
mov word [bx+1+2],ax
mov ax,word [bx-80+1]
div word [bx+80+1]
mov word [bx+1],ax
jmp .END
.PROC2:
mov ax,word [bx-80+1]
sub ax,word [bx+80+1]
mov word [bx+1],ax
mov ax,word [bx-80+1+2]
sbb ax,word [bx+80+1+2]
jz .EQUAL
jc .RESET
mov word [bx+1+2],ax
jmp .END
.EQUAL:
mov word [bx+1],1
mov word [bx+1+2],0
jmp .END
.RESET:
mov word [bx+1],0
mov word [bx+1+2],0
.END:
retn
.ERR;
jmp ASM_ERROR.50

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;       BINARY + -
BINARY_PLUS_MINUS:
mov bx,BUFFER+80
.LOOP:
cmp byte [bx],_END_
je .END
cmp byte [bx],_PLUS_
je .OK
cmp byte [bx],_MINUS_
je .OK
.SKIP:
add bx,80
jmp .LOOP
.OK:
mov al,byte [LEVEL]
cmp byte [bx+15],al
jne .SKIP
mov al,byte [bx-80+15]
mov ah,byte [bx+80+15]
cmp al,ah
jne .SKIP
mov al,byte [bx-80]
mov ah,byte [bx+80]
cmp ax,_NUMBER_+_NUMBER_*256
jne .SKIP
cmp byte [bx],_PLUS_
jne .MINUS
.PLUS:
mov ax,word [bx+80+1]
add word [bx-80+1],ax
mov ax,word [bx+80+1+2]
adc word [bx-80+1+2],ax
jmp .DEL
.MINUS:
mov ax,word [bx+80+1]
sub word [bx-80+1],ax
mov ax,word [bx+80+1+2]
sbb word [bx-80+1+2],ax
.DEL:
mov byte [bx],_INVALID_
mov byte [bx+80],_INVALID_
call RESORTOR
jmp .LOOP
.END:
retn

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;       BINARY <<
BINARY_LSHIFT:
mov bx,BUFFER+80
.LOOP:
cmp byte [bx],_END_
je .END
cmp byte [bx],_LSHIFT_
je .OK
.SKIP:
add bx,80
jmp .LOOP
.OK:
mov al,byte [LEVEL]
cmp byte [bx+15],al
jne .SKIP
mov al,byte [bx-80+15]
mov ah,byte [bx+80+15]
cmp al,ah
jne .SKIP
mov al,byte [bx-80]
mov ah,byte [bx+80]
cmp ax,_NUMBER_+_NUMBER_*256
jne .SKIP
cmp word [bx+80+1+2],0
jne .ERR
cmp word [bx+80+1],0
je .ERR
cmp word [bx+80+1],31
ja .ERR
.ROLL:
shl word [bx-80+1],1
rcl word [bx-80+1+2],1
sub word [bx+80+1],1
jnz .ROLL
.DEL:
mov byte [bx],_INVALID_
mov byte [bx+80],_INVALID_
call RESORTOR
jmp .LOOP
.END:
retn
.ERR:
jmp ASM_ERROR.50

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;       BINARY >>
BINARY_RSHIFT:
mov bx,BUFFER+80
.LOOP:
cmp byte [bx],_END_
je .END
cmp byte [bx],_RSHIFT_
je .OK
.SKIP:
add bx,80
jmp .LOOP
.OK:
mov al,byte [LEVEL]
cmp byte [bx+15],al
jne .SKIP
mov al,byte [bx-80+15]
mov ah,byte [bx+80+15]
cmp al,ah
jne .SKIP
mov al,byte [bx-80]
mov ah,byte [bx+80]
cmp ax,_NUMBER_+_NUMBER_*256
jne .ERR
cmp word [bx+80+1+2],0
jne .ERR
cmp word [bx+80+1],0
je .ERR
cmp word [bx+80+1],31
ja .ERR
.ROLL:
shr word [bx-80+1+2],1
rcr word [bx-80+1],1
sub word [bx+80+1],1
jnz .ROLL
.DEL:
mov byte [bx],_INVALID_
mov byte [bx+80],_INVALID_
call RESORTOR
jmp .LOOP
.END:
retn
.ERR:
jmp ASM_ERROR.50

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;       BINARY &
BINARY_AND:
mov bx,BUFFER+80
.LOOP:
cmp byte [bx],_END_
je .END
cmp byte [bx],_AND_
je .OK
.SKIP:
add bx,80
jmp .LOOP
.OK:
mov al,byte [LEVEL]
cmp byte [bx+15],al
jne .SKIP
mov al,byte [bx-80+15]
mov ah,byte [bx+80+15]
cmp al,ah
jne .SKIP
mov al,byte [bx-80]
mov ah,byte [bx+80]
cmp ax,_NUMBER_+_NUMBER_*256
jne .SKIP
mov ax,word [bx+80+1]
and word [bx-80+1],ax
mov ax,word [bx+80+1+2]
and word [bx-80+1+2],ax
.DEL:
mov byte [bx],_INVALID_
mov byte [bx+80],_INVALID_
call RESORTOR
jmp .LOOP
.END:
retn

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;       BINARY ^
BINARY_XOR:
mov bx,BUFFER+80
.LOOP:
cmp byte [bx],_END_
je .END
cmp byte [bx],_XOR_
je .OK
.SKIP:
add bx,80
jmp .LOOP
.OK:
mov al,byte [LEVEL]
cmp byte [bx+15],al
jne .SKIP
mov al,byte [bx-80+15]
mov ah,byte [bx+80+15]
cmp al,ah
jne .SKIP
mov al,byte [bx-80]
mov ah,byte [bx+80]
cmp ax,_NUMBER_+_NUMBER_*256
jne .SKIP
mov ax,word [bx+80+1]
xor word [bx-80+1],ax
mov ax,word [bx+80+1+2]
xor word [bx-80+1+2],ax
.DEL:
mov byte [bx],_INVALID_
mov byte [bx+80],_INVALID_
call RESORTOR
jmp .LOOP
.END:
retn

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;       BINARY |
BINARY_OR:
mov bx,BUFFER+80
.LOOP:
cmp byte [bx],_END_
je .END
cmp byte [bx],_OR_
je .OK
.SKIP:
add bx,80
jmp .LOOP
.OK:
mov al,byte [LEVEL]
cmp byte [bx+15],al
jne .SKIP
mov al,byte [bx-80+15]
mov ah,byte [bx+80+15]
cmp al,ah
jne .SKIP
mov al,byte [bx-80]
mov ah,byte [bx+80]
cmp ax,_NUMBER_+_NUMBER_*256
jne .SKIP
mov ax,word [bx+80+1]
or word [bx-80+1],ax
mov ax,word [bx+80+1+2]
or word [bx-80+1+2],ax
.DEL:
mov byte [bx],_INVALID_
mov byte [bx+80],_INVALID_
call RESORTOR
jmp .LOOP
.END:
retn

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;       ORPHAN NUMBER
ORPHAN_NUMBER:
mov bx,BUFFER+80
.LOOP:
cmp byte [bx],_END_
je .END
cmp byte [bx],_NUMBER_
je .OK
.SKIP:
add bx,80
jmp .LOOP
.OK:
mov byte [bx+15],0
jmp .SKIP
.END:
retn

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;       RESORTOR
RESORTOR:
mov si,BUFFER+80
mov di,BUFFER_TMP
mov ax,0
.LOOP:
cmp byte [si],_INVALID_
je .SKIP
mov cx,40
add ax,cx
rep movsw
cmp byte [si-80],_END_
je .END
jmp .LOOP
.SKIP:
add si,80
jmp .LOOP
.END:
mov si,BUFFER_TMP
mov di,BUFFER+80
mov cx,ax
rep movsw
retn

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;       SAVE LABEL
SAVE_LABEL:
cmp byte [PASS],'1'
jne .PASS2
.PASS1:
mov si,LABEL_CONTENT
mov di,LABEL_BACKUP
mov cx,32
rep movsw
call SAVE_GLOBAL
call EXTEND_LOCAL
call .PROC
jmp .END
.PASS2:
call SAVE_GLOBAL
.END:
retn
.PROC:
mov es,word [LABEL_SEG]
.LOOP:
es cmp byte [0],_END_
je .NOT_FOUND
mov si,LABEL_CONTENT
mov di,0
mov cx,32
rep cmpsw
je .FOUND
mov ax,es
add ax,5
mov es,ax
cmp ax,0xa000
ja .OUT_OF_MEMORY
jmp .LOOP
.NOT_FOUND:
mov si,LABEL_CONTENT
mov di,0
mov cx,32
rep movsw
mov si,LABEL_OFFSET
movsw
movsw
mov ax,0
mov cx,6
rep stosw
mov ax,''
mov cx,40
rep stosw
mov ax,cs
mov es,ax
retn
.FOUND:
mov ax,cs
mov es,ax
jmp ASM_ERROR.65
.OUT_OF_MEMORY:
mov ax,cs
mov es,ax
jmp ASM_ERROR.06

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;       LOAD LABEL
LOAD_LABEL:
cmp byte [PASS],'1'
jne .PASS2
.PASS1:
mov si,LABEL_OFFSET
mov di,bx
inc di
movsw
movsw
jmp .END
.PASS2:
mov si,bx
add si,16
mov di,LABEL_CONTENT
mov cx,32
rep movsw
mov si,LABEL_CONTENT
mov di,LABEL_BACKUP
mov cx,32
rep movsw
call EXTEND_LOCAL
call LABEL_QUERY
jnc .ERR
mov word [bx+1],ax
mov word [bx+1+2],dx
.END:
retn
.ERR:
jmp ASM_ERROR.70

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;       LABEL QUERY
LABEL_QUERY:
mov es,word [LABEL_SEG]
.LOOP:
es cmp byte [0],_END_
je .NOT_FOUND
mov si,LABEL_CONTENT
mov di,0
mov cx,32
rep cmpsw
je .FOUND
mov ax,es
add ax,5
mov es,ax
jmp .LOOP
.FOUND:
es mov ax,word [64]
es mov dx,word [66]
stc
jmp .END
.NOT_FOUND:
clc
.END:
mov cx,cs
mov es,cx
retn

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;       REPLACE LABEL
REPLACE_LABEL:
call EXTEND_LOCAL
.PROC:
mov es,word [LABEL_SEG]
.LOOP:
mov si,LABEL_CONTENT
mov di,0
mov cx,32
rep cmpsw
je .FOUND
mov ax,es
add ax,5
mov es,ax
jmp .LOOP
.FOUND:
mov si,bx
inc si
movsw
movsw
mov ax,cs
mov es,ax
.END:
retn

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;       EXTEND LOCAL
EXTEND_LOCAL:
cmp byte [LABEL_CONTENT],'.'
jne .END
cmp byte [LABEL_GLOBAL],0
je .ERR1
call RESET_BUFFER_TMP
mov si,LABEL_GLOBAL
mov di,BUFFER_TMP
mov cx,64
.LOOP1:
lodsb
cmp al,0
je .OK1
stosb
loop .LOOP1
jmp .ERR2
.OK1:
mov si,LABEL_CONTENT
.LOOP2:
lodsb
cmp al,0
je .OK2
stosb
loop .LOOP2
jmp .ERR2
.OK2:
mov si,BUFFER_TMP
mov di,LABEL_CONTENT
mov cx,32
rep movsw
.END:
retn
.ERR1:
jmp ASM_ERROR.75
.ERR2:
jmp ASM_ERROR.80

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;       SAVE GLOBAL
SAVE_GLOBAL:
cmp byte [LABEL_CONTENT],'.'
je .END
mov si,LABEL_CONTENT
mov di,LABEL_GLOBAL
mov cx,32
rep movsw
.END:
retn

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;       CLOSE AND DELETE FILES
CLOSE_SOURCE_FILE:
mov bx,word [SOURCE_FILE_HANDLE]
mov ah,0x3e
int 0x21
retn
CLOSE_BINARY_FILE:
mov bx,word [BINARY_FILE_HANDLE]
mov ah,0x3e
int 0x21
retn
DELETE_BINARY_FILE:
mov ah,0x41
mov dx,word [BINARY_FILE_NAME]
int 0x21
retn

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;       DB NUMBER
DB_NUMBER:
cmp byte [PASS],'2'
je .PASS2
.PASS1:
add word [BUFFER_TMP_POINTER],1
jmp .END
.PASS2:
mov si,bx
inc si
mov di,word [BUFFER_TMP_POINTER]
movsb
mov word [BUFFER_TMP_POINTER],di
.END:
retn

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;       DW NUMBER
DW_NUMBER:
cmp byte [PASS],'2'
je .PASS2
.PASS1:
add word [BUFFER_TMP_POINTER],2
jmp .END
.PASS2:
mov si,bx
inc si
mov di,word [BUFFER_TMP_POINTER]
movsw
mov word [BUFFER_TMP_POINTER],di
.END:
retn

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;       DD NUMBER
DD_NUMBER:
cmp byte [PASS],'2'
je .PASS2
.PASS1:
add word [BUFFER_TMP_POINTER],4
jmp .END
.PASS2:
mov si,bx
inc si
mov di,word [BUFFER_TMP_POINTER]
movsw
movsw
mov word [BUFFER_TMP_POINTER],di
.END:
retn

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;       DB STRING
DB_STRING:
cmp byte [PASS],'2'
je .PASS2
.PASS1:
mov si,bx
add si,16
.LOOP1:
lodsb
cmp al,0
je .END
add word [BUFFER_TMP_POINTER],1
jmp .LOOP1
.PASS2:
mov si,bx
add si,16
mov di,word [BUFFER_TMP_POINTER]
.LOOP2:
lodsb
cmp al,0
je .END
stosb
add word [BUFFER_TMP_POINTER],1
jmp .LOOP2
.END:
retn

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;       DW STRING
DW_STRING:
cmp byte [PASS],'2'
je .PASS2
.PASS1:
mov si,bx
add si,16
.LOOP1:
lodsw
cmp al,0
je .END
add word [BUFFER_TMP_POINTER],2
cmp ah,0
je .END
jmp .LOOP1
.PASS2:
mov si,bx
add si,16
mov di,word [BUFFER_TMP_POINTER]
.LOOP2:
lodsw
cmp al,0
je .END
stosw
add word [BUFFER_TMP_POINTER],2
cmp ah,0
je .END
jmp .LOOP2
.END:
retn

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;       DD STRING
DD_STRING:
cmp byte [PASS],'2'
je .PASS2
.PASS1:
mov si,bx
add si,16
.LOOP1:
lodsw
cmp al,0
je .END
add word [BUFFER_TMP_POINTER],4
lodsw
cmp al,0
je .END
cmp ah,0
je .END
jmp .LOOP1
.PASS2:
mov si,bx
add si,16
mov di,word [BUFFER_TMP_POINTER]
.LOOP2:
lodsw
cmp al,0
je .END
stosw
add word [BUFFER_TMP_POINTER],4
lodsw
stosw
cmp al,0
je .END
cmp ah,0
je .END
jmp .LOOP2
.END:
retn

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;       DB DW DD SAVE
DB_DW_DD_SAVE:
cmp byte [PASS],'2'
je .PASS2
.PASS1:
cmp word [TIMES_COUNT],0
je .END
mov ax,word [BUFFER_TMP_POINTER]
sub ax,BUFFER_TMP
add word [LABEL_OFFSET],ax
adc word [LABEL_OFFSET+2],0
dec word [TIMES_COUNT]
jmp .PASS1
.PASS2:
cmp word [TIMES_COUNT],0
je .END
mov cx,word [BUFFER_TMP_POINTER]
sub cx,BUFFER_TMP
mov bx,word [BINARY_FILE_HANDLE]
mov dx,BUFFER_TMP
mov ah,0x40
int 0x21
dec word [TIMES_COUNT]
jmp .PASS2
.END:
retn

_ACCUMULATOR:
push bx
add bx,80
cmp byte [bx],_A_REG_
jne .DISMATCHED
cmp byte [bx+80],_END_
jne .DISMATCHED
.MATCHED:
pop bx
mov word [OP_COUNT],1
call CODE_GEN
stc
retn
.DISMATCHED:
pop bx
clc
retn

_IMMEDIATE:
push bx
add bx,80
cmp byte [bx],_POUND_
jne .DISMATCHED
add bx,80
cmp byte [bx],_NUMBER_
je .NUMBER
cmp byte [bx],_STRING_
je .STRING
jmp .DISMATCHED
.MARK:
cmp byte [bx+80],_END_
jne .DISMATCHED
.MATCHED:
mov word [OP_COUNT],2
call CODE_GEN
pop bx
stc
retn
.DISMATCHED:
pop bx
clc
retn
.NUMBER:
.STRING:
cmp byte [bx+1+1],0
jne .DISMATCHED
mov al,byte [bx+1]
mov byte [OPER],al
jmp .MARK

_ZERO_PAGE:
push bx
add bx,80
cmp byte [bx],_NUMBER_
je .NUMBER
cmp byte [bx],_STRING_
je .STRING
jmp .DISMATCHED
.MARK:
cmp byte [bx+80],_END_
jne .DISMATCHED
.MATCHED:
mov word [OP_COUNT],2
call CODE_GEN
pop bx
stc
retn
.DISMATCHED:
pop bx
clc
retn
.NUMBER:
.STRING:
cmp byte [bx+1+1],0
jne .DISMATCHED
mov al,byte [bx+1]
mov byte [OPER],al
jmp .MARK

_ZERO_PAGE_X:
push bx
add bx,80
cmp byte [bx],_NUMBER_
je .NUMBER
cmp byte [bx],_STRING_
je .STRING
jmp .DISMATCHED
.MARK:
add bx,80
cmp byte [bx],_COMMA_
jne .ERR
add bx,80
cmp byte [bx],_X_REG_
jne .DISMATCHED
cmp byte [bx+80],_END_
jne .DISMATCHED
.MATCHED:
mov word [OP_COUNT],2
call CODE_GEN
pop bx
stc
retn
.DISMATCHED:
pop bx
clc
retn
.NUMBER:
.STRING:
cmp byte [bx+1+1],0
jne .DISMATCHED
mov al,byte [bx+1]
mov byte [OPER],al
jmp .MARK
.ERR:
jmp ASM_ERROR.60

_ZERO_PAGE_Y:
push bx
add bx,80
cmp byte [bx],_NUMBER_
je .NUMBER
cmp byte [bx],_STRING_
je .STRING
jmp .DISMATCHED
.MARK:
add bx,80
cmp byte [bx],_COMMA_
jne .ERR
add bx,80
cmp byte [bx],_Y_REG_
jne .DISMATCHED
cmp byte [bx+80],_END_
jne .DISMATCHED
.MATCHED:
mov word [OP_COUNT],2
call CODE_GEN
pop bx
stc
retn
.DISMATCHED:
pop bx
clc
retn
.NUMBER:
.STRING:
cmp byte [bx+1+1],0
jne .DISMATCHED
mov al,byte [bx+1]
mov byte [OPER],al
jmp .MARK
.ERR:
jmp ASM_ERROR.60

_ABSOLUTE:
push bx
add bx,80
cmp byte [bx],_NUMBER_
je .NUMBER
cmp byte [bx],_STRING_
je .STRING
jmp .DISMATCHED
.MARK:
cmp byte [bx+80],_END_
jne .DISMATCHED
.MATCHED:
mov word [OP_COUNT],3
call CODE_GEN
pop bx
stc
retn
.DISMATCHED:
pop bx
clc
retn
.NUMBER:
.STRING:
mov ax,word [bx+1]
mov word [OPER],ax
jmp .MARK

_ABSOLUTE_X:
push bx
add bx,80
cmp byte [bx],_NUMBER_
je .NUMBER
cmp byte [bx],_STRING_
je .STRING
jmp .DISMATCHED
.MARK:
add bx,80
cmp byte [bx],_COMMA_
jne .ERR
add bx,80
cmp byte [bx],_X_REG_
jne .DISMATCHED
cmp byte [bx+80],_END_
jne .DISMATCHED
.MATCHED:
mov word [OP_COUNT],3
call CODE_GEN
pop bx
stc
retn
.DISMATCHED:
pop bx
clc
retn
.NUMBER:
.STRING:
mov ax,word [bx+1]
mov word [OPER],ax
jmp .MARK
.ERR:
jmp ASM_ERROR.60

_ABSOLUTE_Y:
push bx
add bx,80
cmp byte [bx],_NUMBER_
je .NUMBER
cmp byte [bx],_STRING_
je .STRING
jmp .DISMATCHED
.MARK:
add bx,80
cmp byte [bx],_COMMA_
jne .ERR
add bx,80
cmp byte [bx],_Y_REG_
jne .DISMATCHED
cmp byte [bx+80],_END_
jne .DISMATCHED
.MATCHED:
mov word [OP_COUNT],3
call CODE_GEN
pop bx
stc
retn
.DISMATCHED:
pop bx
clc
retn
.NUMBER:
.STRING:
mov ax,word [bx+1]
mov word [OPER],ax
jmp .MARK
.ERR:
jmp ASM_ERROR.60

_PRE_INDEXED_INDIRECT:
push bx
add bx,80
cmp byte [bx],_LBRACK_
jne .DISMATCHED
add bx,80
cmp byte [bx],_NUMBER_
je .NUMBER
cmp byte [bx],_STRING_
je .STRING
jmp .DISMATCHED
.MARK:
add bx,80
cmp byte [bx],_COMMA_
jne .ERR
add bx,80
cmp byte [bx],_X_REG_
jne .DISMATCHED
add bx,80
cmp byte [bx],_RBRACK_
jne .DISMATCHED
cmp byte [bx+80],_END_
jne .DISMATCHED
.MATCHED:
mov word [OP_COUNT],2
call CODE_GEN
pop bx
stc
retn
.DISMATCHED:
pop bx
clc
retn
.NUMBER:
.STRING:
cmp byte [bx+1+1],0
jne .DISMATCHED
mov al,byte [bx+1]
mov byte [OPER],al
jmp .MARK
.ERR:
jmp ASM_ERROR.60

_POST_INDEXED_INDIRECT:
push bx
add bx,80
cmp byte [bx],_LBRACK_
jne .DISMATCHED
add bx,80
cmp byte [bx],_NUMBER_
je .NUMBER
cmp byte [bx],_STRING_
je .STRING
jmp .DISMATCHED
.MARK:
add bx,80
cmp byte [bx],_RBRACK_
jne .DISMATCHED
add bx,80
cmp byte [bx],_COMMA_
jne .ERR
add bx,80
cmp byte [bx],_Y_REG_
jne .DISMATCHED
cmp byte [bx+80],_END_
jne .DISMATCHED
.MATCHED:
mov word [OP_COUNT],2
call CODE_GEN
pop bx
stc
retn
.DISMATCHED:
pop bx
clc
retn
.NUMBER:
.STRING:
cmp byte [bx+1+1],0
jne .DISMATCHED
mov al,byte [bx+1]
mov byte [OPER],al
jmp .MARK
.ERR:
jmp ASM_ERROR.60

_RELATIVE:
push bx
add bx,80
cmp byte [bx],_NUMBER_
je .NUMBER
cmp byte [bx],_STRING_
je .STRING
jmp .DISMATCHED
.MARK:
cmp byte [bx+80],_END_
jne .DISMATCHED
.MATCHED:
mov word [OP_COUNT],2
call CODE_GEN
pop bx
stc
retn
.DISMATCHED:
pop bx
clc
retn
.NUMBER:
.STRING:
mov ax,word [LABEL_OFFSET]
mov dx,word [LABEL_OFFSET+2]
add ax,2
adc dx,0
cmp word [bx+1+2],dx
jne .ERR
cmp byte [bx+1+1],ah
jne .ERR
cmp byte [bx+1],al
jb .DOWN
.UP:
sub byte [bx+1],al
js .ERR
jmp .JUMP_OK
.DOWN:
sub byte [bx+1],al
jns .ERR
.JUMP_OK:
mov al,byte [bx+1]
mov byte [OPER],al
jmp .MARK
.ERR:
jmp ASM_ERROR.45

_IMPLIED:
push bx
cmp byte [bx+80],_END_
jne .DISMATCHED
.MATCHED:
mov word [OP_COUNT],1
call CODE_GEN
pop bx
stc
retn
.DISMATCHED:
pop bx
clc
retn

_INDIRECT:
push bx
add bx,80
cmp byte [bx],_LBRACK_
jne .DISMATCHED
add bx,80
cmp byte [bx],_NUMBER_
je .NUMBER
cmp byte [bx],_STRING_
je .STRING
jmp .DISMATCHED
.MARK:
add bx,80
cmp byte [bx],_RBRACK_
jne .DISMATCHED
cmp byte [bx+80],_END_
jne .DISMATCHED
.MATCHED:
mov word [OP_COUNT],3
call CODE_GEN
pop bx
stc
retn
.DISMATCHED:
pop bx
clc
retn
.NUMBER:
.STRING:
mov ax,word [bx+1]
mov word [OPER],ax
jmp .MARK

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;       CODE GEN
CODE_GEN:
cmp byte [PASS],'2'
je .PASS2

.PASS1:
cmp word [TIMES_COUNT],0
je .END
mov ax,word [OP_COUNT]
add word [LABEL_OFFSET],ax
adc word [LABEL_OFFSET+2],0
dec word [TIMES_COUNT]
jmp .PASS1

.PASS2:
cmp word [TIMES_COUNT],0
je .END
mov ah,0x40
mov bx,word [BINARY_FILE_HANDLE]
mov cx,word [OP_COUNT]
mov dx,OP
int 0x21
mov ax,word [OP_COUNT]
add word [LABEL_OFFSET],ax
adc word [LABEL_OFFSET+2],0
dec word [TIMES_COUNT]
jmp .PASS2

.END:
retn

[SEGMENT .data]
TITLE           db 'MOS6502 Version 1.00', 13, 10
                db 'Copyright (C) 1997-2001 BTTR Software', 13, 10
                db '[Under GNU GENERAL PUBLIC LICENSE]', 13, 10
                db 13, 10
                db 'MOS6502 comes with ABSOLUTELY NO WARRANTY.', 13, 10
                db 'This is free software, and you are welcome to redistribute it', 13, 10
                db 'under certain conditions.', 13, 10
                db 13, 10, 0
LINE_NO         db 0,0,0,0,0,0,0,1

MESSAGE:
.00 db 'usage: MOS6502 <sourcefilename> <binaryfilename>',0
.01 db 'open source file failed',0
.02 db 'source file is too large',0
.03 db 'create bin file failed',0
.04 db 'load source file failed',0
.05 db 'expecting adequate pairs of conditional definition statements',0
.06 db 'not enough memory space to create symbol table',0
.07 db 'invalid conditional definition statement',0
.10 db 'out of 256 characters statement limitation',0
.15 db 'out of 63 characters identifier limitation',0
.20 db 'label and/or instruction expected at start of line',0
.25 db 'invalid directive combination',0
.30 db 'expression syntax error',0
.35 db 'numeric constant does not fit in 16 bits',0
.40 db 'expecting adequate pairs of quotes',0
.45 db 'short jump is out of range',0
.50 db 'invalid operand',0
.55 db 'invalid opcode and operand(s) combination',0
.60 db 'comma or end of line expected',0
.65 db 'duplicated label redefined',0
.70 db 'symbol not found',0
.75 db 'attemp to use local label before global label',0
.80 db 'out of 63 characters label length limitation',0
.85 db 'attempt to allocate initialized memory in bss section',0
.90 db 'attempt to allocate uninitialized memory in code section',0
.95 db 'expecting colon after label',0
.100 db 'expecting adequate pairs of parenthesis',0
.105 db 'invalid parenthesis sequence',0

KEYWORD:
.OPCODES:
db 3,'adc',3,'and',3,'asl',3,'bcc',3,'bcs',3,'beq',3,'bit',3,'bmi'
db 3,'bne',3,'bpl',3,'brk',3,'bvc',3,'bvs',3,'clc',3,'cld',3,'cli'
db 3,'clv',3,'cmp',3,'cpx',3,'cpy',3,'dec',3,'dex',3,'dey',3,'eor'
db 3,'inc',3,'inx',3,'iny',3,'jmp',3,'jsr',3,'lda',3,'ldx',3,'ldy'
db 3,'lsr',3,'nop',3,'ora',3,'pha',3,'php',3,'pla',3,'plp',3,'rol'
db 3,'ror',3,'rti',3,'rts',3,'sbc',3,'sec',3,'sed',3,'sei',3,'sta'
db 3,'stx',3,'sty',3,'tax',3,'tay',3,'tsx',3,'txa',3,'txs',3,'tya'
.PSEUDO_OPCODES:
db 3,'equ',2,'db',2,'dw',2,'dd',4,'resb',4,'resw',4,'resd',0
.DIRECTIVES:
db 5,'times',3,'bss',3,'org',0
.CONDITIONAL_DEFINITION_OPCODES:
db 5,'ifdef',6,'ifndef',5,'endif',0
.REGISTERS:
db 1,'a',1,'x',1,'y',0

[SEGMENT .bss]
SOURCE_FILE_NAME   resb 2
SOURCE_FILE_HANDLE resb 2
BINARY_FILE_NAME   resb 2
BINARY_FILE_HANDLE resb 2
resb 10
SOURCE_FILE_POS    resb 4
resb 10
SOURCE_FILE_SIZE   resb 4

PASS          resb 1
BSS           resb 1
DEF_CONDITION resb 1
IGNORE        resb 1
TIMES_COUNT   resb 2
LABEL_SEG     resb 2
LABEL_OFFSET  resb 4
LABEL_CONTENT resb 64
LABEL_GLOBAL  resb 64
LABEL_BACKUP  resb 64

STATEMENT_BUF      resb 256
STATEMENT_POINTER  resb 2

TYPE            resb 1
VALUE           resb 14
LEVEL           resb 1
TOKEN_BUF       resb 64

OP       resb 1
OPER     resb 2
OP_COUNT resb 2

BUFFER_POINTER     resb 2
BUFFER             resb 260*80
BUFFER_TMP_POINTER resb 2
BUFFER_TMP         resb 64
